/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Author:
 * Peter van den Bosch
 * pvdb@dse.nl
 * www.dse.nl/~pvdb
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/io.h>
#include "avr-isp.h"
#include "gui.h"


void *nullfunc(int p1, double p2)
{
   return NULL;
}

int
main (int argc, char *argv[])
{
#ifdef GUI
   if (gui_init ())
   {
      avr_run();
      return -1;
   }
   avr_run();
#else /* GUI */
   FILE *fpflash = NULL, *fpeeprom = NULL;
   char buf[1024];

   int i, j;
   unsigned char fuses[3];
   int argcnt;

   argcnt = argc;

   while (--argc)
   {
      if (strcasecmp (argv[argc], "--eeprom") == 0)
      {
         if (argc + 1 < argcnt)
         {
            fpeeprom = fopen (argv[argc + 1], "r");
            FLAG_WRITEAVR = 1;
         }
         else
         {
            terminate (-1, "Please, supply filename");
         }
      }
      if (strcasecmp (argv[argc], "--flash") == 0)
      {
         if (argc + 1 < argcnt)
         {
            fpflash = fopen (argv[argc + 1], "r");
            FLAG_WRITEAVR = 1;
         }
         else
         {
            terminate (-1, "Please, supply filename");
         }
      }
      if (strcasecmp (argv[argc], "--endian") == 0)
      {
#ifdef	ENDIAN
         FLAG_ENDIAN = 0;
#else
         FLAG_ENDIAN = 1;
#endif
      }
      if (strcasecmp (argv[argc], "--help") == 0)
      {
         FLAG_HELP = 1;
      }
      if (strcasecmp (argv[argc], "--list") == 0)
      {
         FLAG_LIST = 1;
      }
      if (strcasecmp (argv[argc], "--version") == 0)
      {
         FLAG_VERSION = 1;
      }
      if (strcasecmp (argv[argc], "--noverify") == 0)
      {
         FLAG_NOVERIFY = 1;
      }
      if (strcasecmp (argv[argc], "--read") == 0)
      {
         FLAG_READAVR = 1;
      }
      if (strcasecmp (argv[argc], "--readfuse") == 0)
      {
         FLAG_READFUSE = 1;
      }
      if (strcasecmp (argv[argc], "--writefuse") == 0)
      {
         if (argc + 2 < argcnt)
         {
            fuses[0] = (unsigned char) strtol (argv[argc + 1], NULL, 16);
            fuses[1] = (unsigned char) strtol (argv[argc + 2], NULL, 16);
            FLAG_WRITEFUSE = 1;
         }
         else
         {
            terminate (-1, "Please, enter values for the fuse-bytes!");
         }
      }
      if (strcasecmp (argv[argc], "--writelock") == 0)
      {
         if (argc + 1 < argcnt)
         {
            fuses[2] = (unsigned char) strtol (argv[argc + 1], NULL, 16);
            FLAG_WRITELOCK = 1;
         }
         else
         {
            terminate (-1, "Please, enter value for the lock-byte!");
         }
      }
   }
   if (FLAG_ENDIAN)
   {
      fprintf (stdout, "You are using endian reversed bytes!\n");
   }
   if (FLAG_VERSION)
   {
      fprintf (stdout, "Version %d.%d%s\n", VERSION_MAJOR,
               VERSION_MINOR, VERSION_RELEASE);
      return 0;
   }
   if (FLAG_HELP)
   {
      fprintf (stdout, HELPMSG);
      return 0;
   }
   if (FLAG_LIST)
   {
      fprintf (stdout, "List of supported devices:\n");
      for (i = 0; id_codes[i].ID; i++)
         fprintf (stdout,
                  "\t%s\t( %4d bytes FLASH, %4d bytes EEPROM)\n",
                  id_codes[i].device, id_codes[i].FLASHsize,
                  id_codes[i].EEPROMsize);
      return 0;
   }
   if (fpflash == NULL && fpeeprom == NULL && FLAG_WRITEAVR)
   {
      fprintf (stderr,
               "No input file: %s --help to display help message.\n", argv[0]);
      return -1;
   }

   /* Set permissions of the IO ports so that we can read & write from and to 
    * it. */
   if (avr_openport () == -1)
   {
      return -1;
   }

   /* Initialize avr and read identification bytes */
   if (avr_init ())
   {
      terminate (-1, "Couldn't sync with device.");
   }
   if( avr_readID( &this_device))
   {
      terminate (-1, "Couldn't read ID from device.");
   } else {
      printf( "Found device %s.\n", this_device->device);
   }

   /* Chip Erase if programming is requested. */
   if (FLAG_WRITEAVR)
   {
      if (avr_erase ())
      {
         terminate (-1, "Failed to erase chip or sync.");
      }
      if( avr_program( fpflash, fpeeprom, nullfunc))
      {
         terminate (-1, "Failed to program AVR.");
      }
   }

   /* If asked, read fuse-bytes */
   if (FLAG_WRITEFUSE)
   {
      fprintf (stdout, "Setting fuses to 0x%X 0x%X (low,high)\n",
               fuses[0], fuses[1]);
      avr_writefuses (fuses);
      FLAG_READFUSE = 1;
   }

   /* If asked, read fuse-bytes */
   if (FLAG_READFUSE)
   {
      int i;

      avr_readfuses (fuses);
      fprintf (stdout, "Fuse bytes: 0x%X 0x%X (low,high)\n",
               fuses[0], fuses[1]);
      if (this_device->fusename_l != 0)
      {
         for (i = 0; i < 8; i++)
         {
            fprintf (stdout,
                     "%-8.8s:  %1d (%.3s)      %-8.8s:  %1d (%.3s)\n",
                     this_device->fusename_l[i],
                     !!(fuses[0] & (0x80 >> i)),
                     fuses[0] & (0x80 >> i) ? "OFF" :
                     "ON ", this_device->fusename_h[i],
                     !!(fuses[1] & (0x80 >> i)),
                     fuses[1] & (0x80 >> i) ? "OFF" : "ON ");
         }
      }
      fprintf (stdout, "\nLock byte:  0x%X\n", fuses[2]);
   }

   if (FLAG_READAVR)
   {                            /* Reading of AVR is requested. */
      avr_init();
      fprintf (stdout, "Reading FLASH:\n");
      for (i = 0; i < this_device->FLASHsize; i += 16)
      {
         avr_read (buf, i, 16, FLASH);
         buf[16] = (unsigned char) (16 + i + 0);   // Checksum
         fprintf (stdout, ":%02X%04X00", 16, i);
         for (j = 0; j < 16; j++)
         {
            fprintf (stdout, " %02X", (unsigned char) buf[j]);
            buf[16] = (unsigned char) buf[j] + (unsigned char) buf[16];
         }
         fprintf (stdout, " -> %02X\n", (unsigned char) (-1 * buf[16]));
      }
      fprintf (stdout, ":00000001FF\n");

      fprintf (stdout, "Reading EEPROM:\n");
      for (i = 0; i < this_device->EEPROMsize; i += 16)
      {
         avr_read (buf, i, 16, EEPROM);
         buf[16] = (unsigned char) (16 + (char) (i / 256) + (char) (i % 256) + 0);  // Checksum
         fprintf (stdout, ":%02X%04X00", 16, i);
         for (j = 0; j < 16; j++)
         {
            fprintf (stdout, " %02X", (unsigned char) buf[j]);
            buf[16] = (unsigned char) buf[j] + (unsigned char) buf[16];
         }
         fprintf (stdout, " -> %02X\n", (unsigned char) (-1 * buf[16]));
      }
      fprintf (stdout, ":00000001FF\n");

      avr_run();
   }

   /* Close file pointers */
   if (fpeeprom != NULL)
   {
      fclose (fpeeprom);
   }
   if (fpflash != NULL)
   {
      fclose (fpflash);
   }

#endif /* GUI */
   return 0;
}
